"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildSchema = exports.processedEntity = exports.entity = void 0;
var mergeRecurseive_1 = require("./utils/mergeRecurseive");
var identity_1 = require("./utils/identity");
var EntitySchema = /** @class */ (function () {
    function EntitySchema(name, idFunction, props, schemas, processFunction) {
        var _a;
        this.nameProp = name;
        this.idFunction = idFunction;
        this.props = props;
        this.schemas = __assign(__assign({}, schemas), (_a = {}, _a[name] = this, _a));
        this.processFunction = processFunction;
    }
    EntitySchema.prototype.normalizeWith = function (input, parent, objectKey) {
        var _a, _b;
        var _this = this;
        var processedInput = this.processFunction(input, parent, objectKey);
        // Initialize the entities object to ensure that all entity types are defined, even if empty
        var baseEntities = Object.keys(this.schemas).reduce(function (entities, nextKey) {
            var _a;
            return (__assign(__assign({}, entities), (_a = {}, _a[nextKey] = {}, _a)));
        }, {});
        var _c = Object.entries(processedInput).reduce(function (_a, _b) {
            var _c, _d, _e;
            var entities = _a.entities, partialObject = _a["processedObject"];
            var key = _b[0], value = _b[1];
            if (key in _this.props) {
                var propType = _this.props[key];
                var schema_1 = _this.schemas[propType];
                if (Array.isArray(value)) {
                    var _f = value.reduce(function (_a, nextVal) {
                        var keys = _a.keys, currentEntities = _a["entities"];
                        var _b = schema_1.normalizeWith(nextVal, processedInput, key), result = _b.result, keyEntities = _b["entities"];
                        return {
                            "keys": keys.concat([result]),
                            // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
                            "entities": mergeRecurseive_1.mergeRecursive(currentEntities, keyEntities)
                        };
                    }, {
                        "keys": [],
                        "entities": {}
                    }), arrayIds = _f["keys"], arrayEntities = _f["entities"];
                    return {
                        // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
                        "entities": arrayEntities,
                        "processedObject": __assign(__assign({}, partialObject), (_c = {}, _c[key] = arrayIds, _c))
                    };
                }
                var _g = schema_1.normalizeWith(value, processedInput, key), result = _g.result, keyEntities = _g["entities"];
                // merge the current entities with the results of the normalize
                return {
                    // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
                    "entities": mergeRecurseive_1.mergeRecursive(entities, keyEntities),
                    "processedObject": __assign(__assign({}, partialObject), (_d = {}, _d[key] = result, _d))
                };
            }
            return {
                entities: entities,
                "processedObject": __assign(__assign({}, partialObject), (_e = {}, _e[key] = value, _e))
            };
        }, { "entities": baseEntities, "processedObject": {}
        }), subEntities = _c["entities"], processedObject = _c.processedObject;
        var thisId = this.idFunction(processedInput, parent, objectKey);
        return {
            "result": thisId,
            "entities": mergeRecurseive_1.mergeRecursive(subEntities, (_a = {},
                _a[this.nameProp] = (_b = {},
                    _b[thisId] = processedObject,
                    _b),
                _a))
        };
    };
    EntitySchema.prototype.normalize = function (input) {
        return this.normalizeWith(input, null, undefined);
    };
    return EntitySchema;
}());
var EntityBuilder = /** @class */ (function () {
    function EntityBuilder(id, name, props, schemas, processFunction) {
        this.idFunction = id;
        this.nameProp = name;
        this.props = props;
        this.schemas = schemas;
        this.processFunction = processFunction;
    }
    EntityBuilder.prototype.name = function (name) {
        return new EntityBuilder(this.idFunction, name, this.props, this.schemas, this.processFunction);
    };
    EntityBuilder.prototype.id = function (idProp) {
        return new EntityBuilder(function (input) { return "" + input[idProp]; }, this.nameProp, this.props, this.schemas, this.processFunction);
    };
    EntityBuilder.prototype.computeId = function (idFunction) {
        return new EntityBuilder(idFunction, this.nameProp, this.props, this.schemas, this.processFunction);
    };
    EntityBuilder.prototype.prop = function (propName, propType) {
        var _a;
        return new EntityBuilder(this.idFunction, this.nameProp, __assign(__assign({}, this.props), (_a = {}, _a[propName] = propType, _a)), this.schemas, this.processFunction);
    };
    EntityBuilder.prototype.define = function (entityName, schema) {
        var _a;
        return new EntityBuilder(this.idFunction, this.nameProp, this.props, __assign(__assign({}, this.schemas), (_a = {}, _a[entityName] = schema, _a)), this.processFunction);
    };
    return EntityBuilder;
}());
function entity() {
    return new EntityBuilder(null, null, {}, {}, identity_1.identity);
}
exports.entity = entity;
function processedEntity(processFunction) {
    return new EntityBuilder(null, null, {}, {}, processFunction);
}
exports.processedEntity = processedEntity;
function buildSchema(builder) {
    return new EntitySchema(builder.nameProp, builder.idFunction, builder.props, builder.schemas, builder.processFunction);
}
exports.buildSchema = buildSchema;
